@@ -28,12 +28,14 @@ const (
 	k8sInClusterCA          = "/var/run/secrets/kubernetes.io/serviceaccount/ca.crt"
 	k8sInClusterBearerToken = "/var/run/secrets/kubernetes.io/serviceaccount/token"
 
-	// CA bundle for cluster-created certificates in OpenShift
-	// https://docs.openshift.org/latest/dev_guide/secrets.html#service-serving-certificate-secrets
-	openshiftInClusterServiceCA = "/var/run/secrets/kubernetes.io/serviceaccount/service-ca.crt"
-
 	// Well-known location of Prometheus service for OpenShift. This is only accessible in-cluster.
 	openshiftPrometheusHost = "prometheus-k8s.openshift-monitoring.svc:9091"
+
+	// The tenancy service port (9092) performs RBAC checks for namespace-specific queries.
+	openshiftPrometheusTenancyHost = "prometheus-k8s.openshift-monitoring.svc:9092"
+
+	// Well-known location of Alert Manager service for OpenShift. This is only accessible in-cluster.
+	openshiftAlertManagerHost = "alertmanager-main.openshift-monitoring.svc:9094"
 )
 
 func main() {
@@ -47,7 +49,8 @@ func main() {
 	fBasePath := fs.String("base-path", "/", "")
 	fConfig := fs.String("config", "", "The YAML config file.")
 
-	fTectonicClusterName := fs.String("tectonic-cluster-name", "tectonic", "The Tectonic cluster name.")
+	// See https://github.com/openshift/service-serving-cert-signer
+	fServiceCAFile := fs.String("service-ca-file", "", "CA bundle for OpenShift services signed with the service signing certificates.")
 
 	fUserAuth := fs.String("user-auth", "disabled", "disabled | oidc | openshift")
 	fUserAuthOIDCIssuerURL := fs.String("user-auth-oidc-issuer-url", "", "The OIDC/OAuth2 issuer URL.")
@@ -79,10 +82,8 @@ func main() {
 	fKubectlClientSecretFile := fs.String("kubectl-client-secret-file", "", "File containing the OAuth2 client_secret of kubectl.")
 	fK8sPublicEndpoint := fs.String("k8s-public-endpoint", "", "Endpoint to use when rendering kubeconfigs for clients. Useful for when bridge uses an internal endpoint clients can't access for communicating with the API server.")
 
-	fDeveloperConsoleURL := fs.String("developer-console-url", "", "URL for the developer console used in masthead context switcher. This option will be removed in a future release.")
-
 	fDexAPIHost := fs.String("dex-api-host", "", "Target host and port of the Dex API service.")
-	fBranding := fs.String("branding", "okd", "Console branding for the masthead logo and title. One of okd, ocp, online, dedicated, or azure. Defaults to okd.")
+	fBranding := fs.String("branding", "okd", "Console branding for the masthead logo and title. One of okd, openshift, ocp, online, dedicated, or azure. Defaults to okd.")
 	fDocumentationBaseURL := fs.String("documentation-base-url", "", "The base URL for documentation links.")
 	fGoogleTagManagerID := fs.String("google-tag-manager-id", "", "Google Tag Manager ID. External analytics are disabled if this is not set.")
 
@@ -136,22 +137,19 @@ func main() {
 		documentationBaseURL = validateFlagIsURL("documentation-base-url", *fDocumentationBaseURL)
 	}
 
-	if *fDeveloperConsoleURL != "" && !strings.HasSuffix(*fDeveloperConsoleURL, "/") {
-		flagFatalf("developer-console-url", "value must end with slash")
-	}
-
 	branding := *fBranding
 	if branding == "origin" {
 		branding = "okd"
 	}
 	switch branding {
 	case "okd":
+	case "openshift":
 	case "ocp":
 	case "online":
 	case "dedicated":
 	case "azure":
 	default:
-		flagFatalf("branding", "value must be one of okd, ocp, online, dedicated, or azure")
+		flagFatalf("branding", "value must be one of okd, openshift, ocp, online, dedicated, or azure")
 	}
 
 	srv := &server.Server{
@@ -160,8 +158,6 @@ func main() {
 		BaseURL:              baseURL,
 		LogoutRedirect:       logoutRedirect,
 		TectonicCACertFile:   caCertFilePath,
-		ClusterName:          *fTectonicClusterName,
-		DeveloperConsoleURL:  *fDeveloperConsoleURL,
 		Branding:             branding,
 		DocumentationBaseURL: documentationBaseURL,
 		GoogleTagManagerID:   *fGoogleTagManagerID,
@@ -190,17 +186,8 @@ func main() {
 
 	var (
 		// Hold on to raw certificates so we can render them in kubeconfig files.
-		dexCertPEM []byte
 		k8sCertPEM []byte
 	)
-	if *fCAFile != "" {
-		var err error
-
-		if dexCertPEM, err = ioutil.ReadFile(*fCAFile); err != nil {
-			log.Fatalf("Failed to read cert file: %v", err)
-		}
-
-	}
 
 	var (
 		k8sAuthServiceAccountBearerToken string
@@ -257,25 +244,31 @@ func main() {
 		k8sAuthServiceAccountBearerToken = string(bearerToken)
 
 		// If running in an OpenShift cluster, set up a proxy to the prometheus-k8s serivce running in the openshift-monitoring namespace.
-		if _, err := os.Stat(openshiftInClusterServiceCA); err == nil {
-			serviceCertPEM, err := ioutil.ReadFile(openshiftInClusterServiceCA)
+		if *fServiceCAFile != "" {
+			serviceCertPEM, err := ioutil.ReadFile(*fServiceCAFile)
 			if err != nil {
 				log.Fatalf("failed to read service-ca.crt file: %v", err)
 			}
-			prometheusProxyRootCAs := x509.NewCertPool()
-			if !prometheusProxyRootCAs.AppendCertsFromPEM(serviceCertPEM) {
+			monitoringProxyRootCAs := x509.NewCertPool()
+			if !monitoringProxyRootCAs.AppendCertsFromPEM(serviceCertPEM) {
 				log.Fatalf("no CA found for Kubernetes services")
 			}
-			prometheusTLSConfig := &tls.Config{RootCAs: prometheusProxyRootCAs}
-			// Only proxy requests to the Prometheus API, not the UI.
+			monitoringProxyTLSConfig := &tls.Config{RootCAs: monitoringProxyRootCAs}
 			srv.PrometheusProxyConfig = &proxy.Config{
-				TLSClientConfig: prometheusTLSConfig,
+				TLSClientConfig: monitoringProxyTLSConfig,
 				HeaderBlacklist: []string{"Cookie", "X-CSRFToken"},
 				Endpoint:        &url.URL{Scheme: "https", Host: openshiftPrometheusHost, Path: "/api"},
 			}
-		} else if !os.IsNotExist(err) {
-			// Ignore errors when the file does not exist, which is the case if not running on OpenShift. Fail on other errors.
-			log.Fatalf("failed to stat service-ca.crt file: %v", err)
+			srv.PrometheusTenancyProxyConfig = &proxy.Config{
+				TLSClientConfig: monitoringProxyTLSConfig,
+				HeaderBlacklist: []string{"Cookie", "X-CSRFToken"},
+				Endpoint:        &url.URL{Scheme: "https", Host: openshiftPrometheusTenancyHost, Path: "/api"},
+			}
+			srv.AlertManagerProxyConfig = &proxy.Config{
+				TLSClientConfig: monitoringProxyTLSConfig,
+				HeaderBlacklist: []string{"Cookie", "X-CSRFToken"},
+				Endpoint:        &url.URL{Scheme: "https", Host: openshiftAlertManagerHost, Path: "/api"},
+			}
 		}
 
 	case "off-cluster":
@@ -367,7 +360,7 @@ func main() {
 
 			// Use the k8s CA file for OpenShift OAuth metadata discovery.
 			// This might be different than IssuerCA.
-			DiscoveryCA: caCertFilePath,
+			K8sCA: caCertFilePath,
 
 			ErrorURL:   authLoginErrorEndpoint,
 			SuccessURL: authLoginSuccessEndpoint,
@@ -384,60 +377,16 @@ func main() {
 			// Assume kubectl is the client ID trusted by kubernetes, not bridge.
 			// These additional flags causes Dex to issue an ID token valid for
 			// both bridge and kubernetes.
-			//
-			// For design see: https://github.com/coreos-inc/tectonic/blob/master/docs-internal/tectonic-identity.md
 			oidcClientConfig.Scope = append(
 				oidcClientConfig.Scope,
 				"audience:server:client_id:"+*fUserAuthOIDCClientID,
 				"audience:server:client_id:"+*fKubectlClientID,
 			)
 
-			kubectlClientSecret := *fKubectlClientSecret
-			if *fKubectlClientSecretFile != "" {
-				buf, err := ioutil.ReadFile(*fKubectlClientSecretFile)
-				if err != nil {
-					log.Fatalf("Failed to read client secret file: %v", err)
-				}
-				kubectlClientSecret = string(buf)
-			}
-
-			// Configure an OpenID Connect config for kubectl. This lets us issue
-			// refresh tokens that kubectl can redeem using its own credentials.
-			kubectlAuthConfig := &auth.Config{
-				IssuerURL:    userAuthOIDCIssuerURL.String(),
-				IssuerCA:     *fCAFile,
-				ClientID:     *fKubectlClientID,
-				ClientSecret: kubectlClientSecret,
-				// The magic "out of band" redirect URL.
-				RedirectURL: "urn:ietf:wg:oauth:2.0:oob",
-				// Request a refresh token with the "offline_access" scope.
-				Scope: []string{"openid", "email", "profile", "offline_access", "groups"},
-
-				ErrorURL:   authLoginErrorEndpoint,
-				SuccessURL: authLoginSuccessEndpoint,
-
-				CookiePath:    cookiePath,
-				RefererPath:   refererPath,
-				SecureCookies: secureCookies,
-			}
-
-			if srv.KubectlAuther, err = auth.NewAuthenticator(context.Background(), kubectlAuthConfig); err != nil {
-				log.Fatalf("Error initializing kubectl authenticator: %v", err)
-			}
-
-			srv.KubeConfigTmpl = server.NewKubeConfigTmpl(
-				*fTectonicClusterName,
-				*fKubectlClientID,
-				*fKubectlClientSecret,
-				apiServerEndpoint,
-				userAuthOIDCIssuerURL.String(),
-				k8sCertPEM,
-				dexCertPEM,
-			)
 		}
 
 		if srv.Auther, err = auth.NewAuthenticator(context.Background(), oidcClientConfig); err != nil {
-			log.Fatalf("Error initializing OIDC authenticator: %v", err)
+			log.Fatalf("Error initializing authenticator: %v", err)
 		}
 	case "disabled":
 		log.Warningf("running with AUTHENTICATION DISABLED!")
