@@ -5,6 +5,19 @@
  */
 /* eslint-disable no-console */
 
+import { getCSRFToken } from '../co-fetch';
+
+function addCSRFQueryParam(href) {
+  const csrfToken = getCSRFToken();
+  if (!csrfToken) {
+    return href;
+  }
+
+  const url = new URL(href);
+  url.searchParams.set('x-csrf-token', csrfToken);
+  return url.href;
+}
+
 function createURL(host, path) {
   let url;
 
@@ -23,7 +36,7 @@ function createURL(host, path) {
     url += path;
   }
 
-  return url;
+  return addCSRFQueryParam(url);
 }
 
 export function WSFactory(id, options) {
@@ -99,13 +112,13 @@ WSFactory.prototype._connect = function() {
       that._connectionAttempt = null;
     }
   };
-  this.ws.onclose = function (evt) {
+  this.ws.onclose = function(evt) {
     console.log(`websocket closed: ${that.id}`, evt);
     that._state = 'closed';
     that._triggerEvent('close', evt);
     that._reconnect();
   };
-  this.ws.onerror = function (evt) {
+  this.ws.onerror = function(evt) {
     console.log(`websocket error: ${that.id}`);
     that._state = 'error';
     that._triggerEvent('error', evt);
@@ -268,6 +281,6 @@ WSFactory.prototype.destroy = function(timedout) {
   this._messageBuffer = [];
 };
 
-WSFactory.prototype.send = function (data) {
+WSFactory.prototype.send = function(data) {
   this.ws && this.ws.send(data);
 };
